import inspect
import subprocess as subproc
import sys
from typing import Dict, List, Optional, Set, Tuple, Union, Any

from logfunc import logf


def nlprint(*args, **kwargs) -> None:
    """Identical to normal print() but also prints an extra \n"""
    print(*args, **kwargs)
    print()


def is_jwt_str(s: str) -> bool:
    """Check if a string is an encrypted JWT"""
    parts = str(s).split('.')
    return len(parts) == 3


@logf(level="info")
def runcmd(cmd: str, output: bool = True, *args, **kwargs) -> Optional[List[str]]:
    """
    Run a command in the shell.

    Args:
        cmd (str): The command to be executed.
        output (bool, optional): Specifies whether to capture and return the output of the command.
            Defaults to True.
        *args: Additional positional arguments to be passed to subprocess.run().
        **kwargs: Additional keyword arguments to be passed to subprocess.run().

    Returns:
        Optional[List[str]]: The captured output of the command as a list of lines if `output` is True.
            None if `output` is False.

    Raises:
        CalledProcessError: If the command exits with a non-zero status and `check=True`.

    """
    if output:
        return subproc.run(
            [c for c in cmd.split()],
            check=True,
            text=True,
            capture_output=True,
            *args,
            **kwargs,
        ).stdout.splitlines()
    else:
        subproc.run(
            [c for c in cmd.split()],
            check=False,
            text=False,
            capture_output=False,
            *args,
            **kwargs,
        )


class ObjInfo:
    """
    A class to gather and print properties of a python object.
    """

    obj: Any
    obj_type: str
    obj_attrs: List[str]
    obj_methods: List[str]
    obj_doc: Optional[str]
    obj_scope: str
    obj_size: int
    obj_mutability: str
    obj_identity: int
    info_attrs: List[str] = [
        "obj",
        "obj_type",
        "obj_attrs",
        "obj_methods",
        "obj_doc",
        "obj_scope",
        "obj_size",
        "obj_mutability",
        "obj_identity",
    ]

    def __init__(self, var: Any):
        """
        Initialize with the object to be inspected
        :param var: The object to inspect
        """
        self.obj = var
        self.obj_type = type(var)
        self.obj_name = var.__name__ if hasattr(var, "__name__") else "N/A"

        self.obj_attrs = sorted(
            attr for attr in dir(var) if not callable(getattr(var, attr))
        )
        self.obj_methods = sorted(
            method for method in dir(var) if callable(getattr(var, method))
        )
        self.obj_doc = inspect.getdoc(var)
        self.obj_scope = (
            "Local"
            if var in inspect.currentframe().f_back.f_locals.values()
            else "Global"
        )
        self.obj_size = sys.getsizeof(var)
        self.obj_mutability = "Mutable" if hasattr(var, "__dict__") else "Immutable"
        self.obj_identity = id(var)
        self.print_info()

    def print_info(self):
        """
        Prints the collected information of the object
        """
        curline = "Object: {} {} {} {} {}".format(
            self.obj_name,
            self.obj_type,
            self.obj_mutability,
            self.obj_scope,
            self.obj_identity,
        )
        nlprint(10 * '=', curline, 10 * '=')

        nlprint("Object __repr__/__str__:", self.obj)
        nlprint("Object Attributes:", " ".join(self.obj_attrs))
        nlprint("Object Methods:", " ".join(self.obj_methods))
        print(20 * '-', 'Object Docstring', 20 * '-')
        print("Object Documentation:", str(self.obj_doc))
        nlprint(len(20 * '-' + ' Object Docstring ' + 20 * '-') * '-')



#!/usr/bin/env python3
import logging
import unittest
import sys
from pathlib import Path
from unittest.mock import MagicMock

sys.path.insert(
    0, str(Path(__file__).parent.parent)
)  # Add the parent directory of tests to the system path

from myfuncs.funcs import logf, get_asctime, valid_uuid, trunc_str, ran_str

logging.basicConfig(level=logging.DEBUG)


class TestMyFuncs(unittest.TestCase):
    @logf()
    def test_get_asctime(self):
        asctime = get_asctime()
        self.assertIsNotNone(asctime)
        self.assertIsInstance(asctime, str)

    @logf()
    def test_valid_uuid(self):
        uuid_valid = "123e4567-e89b-12d3-a456-426614174000"
        uuid_invalid = "123e4567-e89b-12d3-a456-42661417400"

        self.assertTrue(valid_uuid(uuid_valid))
        self.assertFalse(valid_uuid(uuid_invalid))

    @logf()
    def test_trunc_str(self):
        s = "abcdefg"
        self.assertEqual(trunc_str(s, 5), "ab...")
        self.assertEqual(trunc_str(s, 7), "abcdefg")
        self.assertEqual(trunc_str(s, 10), "abcdefg")
        self.assertEqual(trunc_str(s, 0), "abcd...")

    @logf()
    def test_ran_str(self):
        s1, s2 = ran_str(32), ran_str(32)
        self.assertEqual(len(s1), 32)
        self.assertEqual(len(s1), len(s2))
        self.assertNotEqual(s1, s2)
        s3 = ran_str(16)
        self.assertEqual(len(s3), 16)


class TestLogF(unittest.TestCase):
    @logf()
    def test_logf_decorator(self):
        logger_mock = MagicMock()

        @logf(level=logging.DEBUG, log_args=True, log_return=True, measure_time=True)
        def example_func(a, b):
            return a + b

        with unittest.mock.patch('myfuncs.funcs.logger', logger_mock):
            result = example_func(1, 2)

        self.assertEqual(result, 3)
        self.assertEqual(logger_mock.log.call_count, 2)
        logger_mock.log.assert_any_call(logging.DEBUG, "example_func() | (1, 2) {}")
        log_message = logger_mock.log.call_args_list[1][0][1]
        self.assertTrue(log_message.startswith("example_func()"))
        self.assertTrue(log_message.endswith(" | 3"))

    @logf()
    def test_logf_no_args_no_return_no_time(self):
        logger_mock = MagicMock()

        @logf(level=logging.DEBUG, log_args=False, log_return=False, measure_time=False)
        def example_func(a, b):
            return a + b

        with unittest.mock.patch('myfuncs.funcs.logger', logger_mock):
            result = example_func(1, 2)

        self.assertEqual(result, 3)
        self.assertEqual(logger_mock.log.call_count, 1)

    @logf()
    def test_logf_max_str_len(self):
        logger_mock = MagicMock()

        @logf(
            level=logging.DEBUG,
            log_args=True,
            log_return=True,
            max_str_len=10,
            measure_time=True,
        )
        def example_func(a, b):
            return "abcde" * 1000  # Return a very long string

        with unittest.mock.patch('myfuncs.funcs.logger', logger_mock):
            result = example_func(
                "abcde" * 1000, 2
            )  # Pass a very long string as an argument

        self.assertEqual(result, "abcde" * 1000)
        self.assertEqual(logger_mock.log.call_count, 2)
        log_message = logger_mock.log.call_args_list[1][0][1]
        self.assertTrue(log_message.startswith("example_func()"))
        self.assertTrue("..." in log_message)  # Log message should be truncated

    @logf()
    def test_logf_different_level(self):
        logger_mock = MagicMock()

        @logf(level=logging.WARNING, log_args=True, log_return=True, measure_time=True)
        def example_func(a, b):
            return a + b

        with unittest.mock.patch('myfuncs.funcs.logger', logger_mock):
            result = example_func(1, 2)

        self.assertEqual(result, 3)
        self.assertEqual(logger_mock.log.call_count, 2)
        logger_mock.log.assert_any_call(logging.WARNING, "example_func() | (1, 2) {}")
        log_message = logger_mock.log.call_args_list[1][0][1]
        self.assertTrue(log_message.startswith("example_func()"))
        self.assertTrue(log_message.endswith(" | 3"))


if __name__ == "__main__":
    unittest.main()



import unittest
from subprocess import CompletedProcess
import sys
from pathlib import Path
import os
from unittest.mock import patch, MagicMock
from os.path import dirname, abspath

MFROOT = str(dirname(dirname(abspath(__file__))))
if MFROOT not in sys.path:
    sys.path.insert(0, MFROOT)

from myfuncs import runcmd, ObjInfo  # noqa


class TestRunCmd(unittest.TestCase):
    def test_runcmd_with_output(self):
        # Mock the subprocess.run() function to return a CompletedProcess object
        mock_completed_process = CompletedProcess(
            args=['echo', 'Hello, World!'],
            returncode=0,
            stdout='Hello, World!\n',
            stderr='',
        )
        with patch('subprocess.run', return_value=mock_completed_process):
            result = runcmd('echo Hello, World!')

        self.assertEqual(result, ['Hello, World!'])

    def test_runcmd_without_output(self):
        # Mock the subprocess.run() function to return None
        with patch('subprocess.run'):
            result = runcmd('echo Hello, World!', output=False)

        self.assertIsNone(result)


class TestObjInfo(unittest.TestCase):
    def test_print_info(self):
        obj = TestObj()
        oinfo = ObjInfo(obj)
        with patch("builtins.print") as mock_print:
            oinfo.print_info()

        mock_print.assert_called()

    def test_obj_info_attributes(self):
        obj = TestObj()
        oinfo = ObjInfo(obj)
        self.assertEqual(oinfo.obj_name, 'objname')
        self.assertEqual(oinfo.obj_type, type(obj))
        for attstr in ["attr1", "attr2"]:
            self.assertTrue(attstr in oinfo.obj_attrs)
        for mstr in ["method1", "method2"]:
            self.assertTrue(mstr in oinfo.obj_methods)
        self.assertEqual(oinfo.obj_doc, "This is the documentation for TestObj")
        self.assertEqual(oinfo.obj_scope, "Local")
        self.assertEqual(oinfo.obj_mutability, "Mutable")
        self.assertEqual(oinfo.obj_identity, id(obj))


class TestObj:
    attr1 = "Attribute 1"
    attr2 = "Attribute 2"

    def method1(self):
        pass

    def method2(self):
        pass

    __doc__ = "This is the documentation for TestObj"
    __name__ = 'objname'


if __name__ == '__main__':
    unittest.main()



"""
legacy file, kept around for off chance somebody is relying
on backwards compatability, all functionality should go in __init__.py
"""
import os
import re
import time
import inspect
import random
import string
from datetime import datetime
from functools import wraps
from typing import Any, Callable, Optional, TypeVar, Union

import logging

logger = logging.getLogger(__name__)

T = TypeVar("T", bound=Callable[..., Any])

ASCII_CHARS = string.ascii_letters + string.ascii_lowercase + string.ascii_uppercase


def trunc_str(string: str, max_length: int) -> str:
    """
    Truncates a string if its length exceeds the specified maximum length.
    If the string is truncated, it appends '...' to indicate the truncation.

    Args:
        string (str): The string to truncate.
        max_length (int): The maximum length of the truncated string.

    Returns:
        str: The truncated string.
    """
    if len(string) > max_length:
        return string[:max_length - 3] + "..."
    return string


def logf(
    level: Optional[Union[int, str]] = logging.DEBUG,
    log_args: bool = True,
    log_return: bool = True,
    max_str_len: int = 1000,
    measure_time: bool = True
) -> Callable[[T], T]:
    """
    A decorator that logs the execution time, function name, arguments, keyword arguments,
    and return value of a function using a specified log level.

    Args:
        level (Union[int, str], optional): The log level to use for logging. Defaults to logging.DEBUG.
        log_args (bool, optional): Should the function arguments be logged? Defaults to True.
        log_return (bool, optional): Should function return be logged? Defaults to True.
        max_str_len (int, optional): Maximum length of the logged arguments and return values. Defaults to 1000.
        measure_time (bool, optional): Should the function execution time be measured? Defaults to True.

    Returns:
        Callable[[T], T]: The wrapped function.
    """
    if isinstance(level, str):
        level_int = logging.getLevelName(level.upper())
    else:
        level_int = int(level)

    def decorator(func: T) -> T:
        @wraps(func)
        def wrapper(*args: Any, **kwargs: Any) -> Any:
            # Start the timer if required and execute the function.
            start_time = time.time() if measure_time else None

            # Log function arguments if required
            if log_args:
                arg_str = f"{func.__name__}() | {str(args)[:max_str_len]} {str(kwargs)[:max_str_len]}"
            else:
                arg_str = f"{func.__name__}()"

            logger.log(level_int, arg_str)

            # Execute the function
            result = func(*args, **kwargs)

            if measure_time:
                end_time = time.time()
                # Calculate the execution time and format the log message.
                exec_time = end_time - start_time
                exec_time_str = f"{exec_time:.5f}s"

            # Log the return value and execution time if required
            if log_return and measure_time:
                result_str = trunc_str(str(result), max_str_len)
                log_message = f"{func.__name__}() {exec_time_str} | {result_str}"
            elif log_return:
                result_str = trunc_str(str(result), max_str_len)
                log_message = f"{func.__name__}() | {result_str}"
            elif measure_time:
                log_message = f"{func.__name__}() {exec_time_str}"
            else:
                log_message = None

            if log_message is not None:
                # Log the message using the specified level.
                logger.log(level_int, log_message)

            return result
        return wrapper
    return decorator


@logf()
def ran_str(length: int) -> str:
    """ Returns a string of random ascii characters of n length

    Args:
        length (int): length of returned str

    Returns:
        str: random char string of n length
    """
    s = ''
    for i in range(0, length):
        s += random.choice(ASCII_CHARS)
    return s



@logf()
def get_asctime() -> str:
    """Returns the current time in the same format as logging %(asctime)s


    Returns:
        str: current asctime
    """
    now = datetime.now()
    return now.strftime(f"%Y-%m-%d %H:%M:%S")


@logf()
def valid_uuid(string):
    regex = re.compile(
        r'^[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[0-9a-f]{4}-[0-9a-f]{12}$')
    return bool(regex.match(string))